// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//  File:    appCircle.asy
//  Author:  Thomas Mueller, University of Stuttgart, VISUS
//
//    Apparent view of a moving circle.
//
// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++

usepackage("amssymb");
//usepackage("mathptmx");

usepackage("format");

import geometry;
import graph;

import sr_apparent;

size(18cm,0);

// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++

// observer position, time, field of view
pair xObs = ( 0.0, -10.0 );
real tObs = 0.0; 

// object velocity
real beta = 0.95;
real gam  = 1.0/sqrt(1-beta^2);

real ballRadius = 1.0;
pair ac = (0,0);

real lwidth = 1bp;

// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++

pair appCenter() {
    triple ax = appPointPos( (0,0,0), (xObs.x,xObs.y,0), (beta,0,0), (0,0,0), tObs );
    return (ax.x,ax.y);
}

pair appCircle( real phi ) {
    triple ax = appSpherePos( (0,0,0), (xObs.x,xObs.y,0), (beta,0,0), (0,0,0), 
                              (1,0,0), (0,1,0), (0,0,1), 
                              ballRadius, pi/2, phi, tObs );
    return (ax.x,ax.y);
}

/*
// simplified version
pair appCenter() {
    real omega2 = gam^2 * beta^2 * tObs^2 + xObs.y^2;     
    real xs0 = gam*tObs - sqrt(omega2);    
    return (gam*xs0*beta,0.0);
}
 
pair appCircle( real phi ) {
    real s1 = ballRadius*cos(phi);
    real s2 = ballRadius*sin(phi);
    real rho = -tObs;
    real mux = gam*tObs*beta;
    real muy = -xObs.y;
    real omega2 = gam^2 * beta^2 * tObs^2 + xObs.y^2 + 2.0*(mux*s1 + muy*s2) + ballRadius^2;    
    real xs0 = gam*tObs - sqrt(omega2);    
    return (gam*xs0*beta + gam*s1,s2);
}
*/
       
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++    
       
// minimal observer canvas       
pair ll = (-19.0,-10.5);
pair ur = (14.5,2.5);
//fill(ll--(ur.x,ll.y)--ur--(ll.x,ur.y)--cycle,white);
//drawBox(MVP);


for(int n=0; n<13; ++n) {
    real posX = -17.5 + n*2.5;
    filldraw(shift(posX,0)*unitcircle,lightgray,gray+dashed);
}

dot(xObs,5bp+blue);
label("observer",(xObs.x+0.1,xObs.y),align=E,blue);


//draw axes
draw(line((ll.x,0),false,(ur.x,0),false),Arrow(2.5mm));
draw(line((0,ll.y),false,(0,ur.y),false),Arrow(2.5mm));
label("$x^1$",(ur.x,0),align=S);
label("$x^2$",(0,ur.y),align=W);

// draw tics
real tsize = 0.1;
real tstep = 1.0;
for(real x=0; x<ur.x-tstep; x+=tstep) {
   draw(line((x,-tsize),false,(x,tsize),false));
}
for(real x=-tstep; x>ll.x; x-=tstep) {
   draw(line((x,-tsize),false,(x,tsize),false));
}
for(real y=0; y<ur.y-tstep; y+=tstep) {
   draw(line((-tsize,y),false,(tsize,y),false));
}
for(real y=-tstep; y>ll.y; y-=tstep) {
   draw(line((-tsize,y),false,(tsize,y),false));
}

tObs = 3.0;
ac = appCenter();
filldraw(graph(appCircle,0,2*pi,50)--cycle,0.2*red+10*white);
dot(ac,3bp+black);
label("$x_{\obs}^0=3.0$",(ac.x,ac.y+1.1),align=N,fontsize(11));


tObs = 6.0;
ac = appCenter();
filldraw(graph(appCircle,0,2*pi,50)--cycle,0.2*red+10*white);
dot(ac,3bp+black);
label("$x_{\obs}^0=6.0$",(ac.x,ac.y+1.1),align=N,fontsize(11));

pair dir = (ac-xObs);
real l = 8.0;
real fovh = 20.0;

real da = -atan(dir.y/dir.x);
draw(xObs--ac,blue+dotted+lwidth);
draw(xObs--(xObs+l*(-cos(da+radians(fovh)),sin(da+radians(fovh)))),blue+dashed+lwidth);
draw(xObs--(xObs+l*(-cos(da-radians(fovh)),sin(da-radians(fovh)))),blue+dashed+lwidth);
draw(arc(xObs,3,180-degrees(da)+fovh,180-degrees(da)-fovh),blue+lwidth);
label("fov",xObs+(-1.0,2.0),blue);



tObs = 10.0;
ac = appCenter();
filldraw(graph(appCircle,0,2*pi,50)--cycle,0.2*red+10*white);
dot(ac,3bp+black);
label("$x_{\obs}^0=10.0$",(ac.x,ac.y+1.1),align=N,fontsize(11));

tObs = 25.0;
ac = appCenter();
filldraw(graph(appCircle,0,2*pi,50)--cycle,0.2*red+10*white);
dot(ac,3bp+black);
label("$x_{\obs}^0=25.0$",(ac.x,ac.y+1.1),align=N,fontsize(11));

