// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//  File:    appCube.asy
//  Author:  Thomas Mueller, University of Stuttgart, VISUS
//
//    Apparent view of a moving cube for close encounter.
//
// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++

usepackage("amssymb");
usepackage("mathptmx");

import geometry;
import graph;

import sr_objects;
import sr_camera;
import sr_apparent;

size(8cm,0);

// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++

// observer position, time, field of view
triple xObs = ( 0.5, 2, 0.05 );
real   tObs = length(xObs);

real   fovY   = radians(32.0);

triple dir    = ((0.0,0.0,-0.3) - xObs);
dir = dir/length(dir);
triple center = dir-xObs;
triple upV    = ( 0.0, 0.0, 1.0 );

// offset between S and S'
triple aOff   = ( 0.0, 0.0, 0.0 );

// object velocity
triple beta = (0.0,0.9,0.0);

// point in S'
triple XS = (0,0,0);

// direction of line
triple sigma = (1,0,0);

// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++


// View-Projection matrix
real[][] projMatrix = getPerspMatrix( fovY, 1.0, 0.1, 100.0 );
real[][] viewMatrix = getViewMatrix( xObs, center, upV );
real[][] MVP = projMatrix*viewMatrix;

// apparent line
pair appLine( real s ) {
    return map(MVP,appLinePos(aOff,xObs,beta,XS,sigma,s,tObs));
}

       
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++    
       
// minimal observer canvas       
pair ll = (-0.9,-0.81);
pair ur = (0.9,1.01);
fill(ll--(ur.x,ll.y)--ur--(ll.x,ur.y)--cycle,white);
//drawBox(MVP);

// blue edges
for(int i=0; i<numBoxEdgesBlue; ++i) {
    XS = 0.5*(boxVerts[boxEdgesBlue[i][0]] - (0.5,0.5,0.5));
    sigma = boxVerts[boxEdgesBlue[i][1]]-boxVerts[boxEdgesBlue[i][0]];
    draw(graph(appLine,0,0.5,30),1.5bp+blue);
}

//red edges
for(int i=0; i<numBoxFrontEdges; ++i) {
    XS = 0.5*(boxFrontVerts[boxFrontEdges[i][0]] - (0.5,0.5,0.5));
    sigma = boxFrontVerts[boxFrontEdges[i][1]]-boxFrontVerts[boxFrontEdges[i][0]];
    draw(graph(appLine,0,0.5,30),1.5bp+red);
}
