// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//  File:    appDie.asy
//  Author:  Thomas Mueller, University of Stuttgart, VISUS
//
//    Apparent view of a moving die.
//
// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++

usepackage("amssymb");
usepackage("mathptmx");

import geometry;
import graph;

import sr_objects;
import sr_camera;
import sr_apparent;

size(8cm,0);

// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++

// observer position, time, field of view
triple xObs = ( 8.0, 12.0, 2.0 );
real   tObs = 14.0;

real   fovY   = radians(32.0);

triple dir    = ((0.0,0.0,-0.3) - xObs);
dir = dir/length(dir);
triple center = dir-xObs;
triple upV    = ( 0.0, 0.0, 1.0 );

// offset between S and S'
triple aOff   = ( 0.0, 0.0, 0.0 );

// object velocity
triple beta = (0.0,0.9,0.0);

// point in S'
triple XS = (0,0,0);

// direction of line
triple sigma = (1,0,0);

real lwidth = 0.7bp;

// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++


// View-Projection matrix
real[][] projMatrix = getPerspMatrix( fovY, 1.0, 0.1, 100.0 );
real[][] viewMatrix = getViewMatrix( xObs, center, upV );
real[][] MVP = projMatrix*viewMatrix;

// apparent line
pair appLine( real s ) {
    return map(MVP,appLinePos(aOff,xObs,beta,XS,sigma,s,tObs));
}

       
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++    
       
// minimal observer canvas       
pair ll = (-1.0,-0.5);
pair ur = (1.0,0.3);
fill(ll--(ur.x,ll.y)--ur--(ll.x,ur.y)--cycle,white);
//drawBox(MVP);

// blue edges
tObs = 14.012;  // 136
for(int i=0; i<numBoxEdgesBlue; ++i) {
    XS = 0.5*(boxVerts[boxEdgesBlue[i][0]] - (0.5,0.5,0.5));
    sigma = boxVerts[boxEdgesBlue[i][1]]-boxVerts[boxEdgesBlue[i][0]];
    draw(graph(appLine,0,0.5,10),lwidth+blue);
}

tObs = 15.1;    // 200
for(int i=0; i<numBoxEdgesBlue; ++i) {
    XS = 0.5*(boxVerts[boxEdgesBlue[i][0]] - (0.5,0.5,0.5));
    sigma = boxVerts[boxEdgesBlue[i][1]]-boxVerts[boxEdgesBlue[i][0]];
    draw(graph(appLine,0,0.5,10),lwidth+blue);
}

tObs = 16.12;  // 260
for(int i=0; i<numBoxEdgesBlue; ++i) {
    XS = 0.5*(boxVerts[boxEdgesBlue[i][0]] - (0.5,0.5,0.5));
    sigma = boxVerts[boxEdgesBlue[i][1]]-boxVerts[boxEdgesBlue[i][0]];
    draw(graph(appLine,0,0.5,10),lwidth+blue);
}

//red edges
tObs = 14.012;  // 136
for(int i=0; i<numBoxFrontEdges; ++i) {
    XS = 0.5*(boxFrontVerts[boxFrontEdges[i][0]] - (0.5,0.5,0.5));
    sigma = boxFrontVerts[boxFrontEdges[i][1]]-boxFrontVerts[boxFrontEdges[i][0]];
    draw(graph(appLine,0,0.5,10),lwidth+red);
}

tObs = 15.1;    // 200
for(int i=0; i<numBoxFrontEdges; ++i) {
    XS = 0.5*(boxFrontVerts[boxFrontEdges[i][0]] - (0.5,0.5,0.5));
    sigma = boxFrontVerts[boxFrontEdges[i][1]]-boxFrontVerts[boxFrontEdges[i][0]];
    draw(graph(appLine,0,0.5,10),lwidth+red);
}

tObs = 16.12;  // 260
for(int i=0; i<numBoxFrontEdges; ++i) {
    XS = 0.5*(boxFrontVerts[boxFrontEdges[i][0]] - (0.5,0.5,0.5));
    sigma = boxFrontVerts[boxFrontEdges[i][1]]-boxFrontVerts[boxFrontEdges[i][0]];
    draw(graph(appLine,0,0.5,10),lwidth+red);
}


beta = (0,0,0);
for(int n=0; n<8; ++n) {
    aOff = (0.0, -10.0 + n*2.0, -0.75 );

    // blue edges
    for(int i=0; i<numBoxEdgesBlue; ++i) {
        XS = 0.5*(boxVerts[boxEdgesBlue[i][0]] - (0.5,0.5,0.5));
        sigma = 0.5*(boxVerts[boxEdgesBlue[i][1]]-boxVerts[boxEdgesBlue[i][0]]);
        draw(graph(appLine,0,1,10),lwidth+blue);
        if(i==0){
            pen fsize = fontsize(10pt);
            label(string(n), appLine(0), align=(1.5+0.25*n)*S+(-0.1*(8-n))*E, fsize);
        }
    }
    
    // red edges
    for(int i=0; i<numBoxFrontEdges; ++i) {
        XS = 0.5*(boxFrontVerts[boxFrontEdges[i][0]] - (0.5,0.5,0.5));
        sigma = 0.5*(boxFrontVerts[boxFrontEdges[i][1]]-boxFrontVerts[boxFrontEdges[i][0]]);
        draw(graph(appLine,0,1,10),lwidth+red);
    }
}
