// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//  File:    appRod.asy
//  Author:  Thomas Mueller, University of Stuttgart, VISUS
//
//    Apparent view of a moving rod -> figure 7
//
// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++

usepackage("amssymb");
//usepackage("mathptmx");

usepackage("format");

import geometry;
import graph;

import sr_apparent;

size(12.5cm);

// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++

// observer position, time
triple xObs = ( 0.0, 0.0, 0.0 );
real   tObs = 1.1;

// offset between S and S'
triple aOff = ( 0.0, 0.0, 0.0 );

// point in S'
triple XS = (0,0,0);

// object velocity
triple beta  = (0.75,0.0,0.0);

// direction of line
triple sigma = (0.0,1.0,0.0);

// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++ 

pair centerOfCircle = (0,0);
real radiusOfCircle = 1;

path circle( pair c, real r ) {
   return shift(c)*scale(r)*unitcircle;
}

pair hyp( real xi ) {
   //return (zeta0 - gam^2*b^2/kappa*sqrt(1+(xi-xi0+mu)^2/(gam^2*b^2/kappa^2)),xi);
    triple ax = appLinePos(aOff,xObs,beta,XS,sigma,xi,tObs);
    return (ax.x,ax.y);
}

void recalcParams() {
   real gam = 1.0/sqrt(1.0-dot(beta,beta));
   real b   = sqrt(dot(beta,beta));
   real g2  = gam*gam/(gam+1.0);
   real rho = gam*dot(XS,beta) - tObs;
   triple eta = XS + g2*dot(XS,beta)*beta + aOff - xObs;
   real   rb  = rho-dot(beta,eta);
   real   mu  = dot(eta,sigma);
   real omega = sqrt(gam*gam*rb*rb - rho*rho + dot(eta,eta) - mu^2);
   
   real zeta0 = gam^2 * b * (tObs + dot(beta,aOff-xObs)) + gam * dot(beta,XS) + dot(aOff,beta);
   real kappa = gam*b/omega;

   centerOfCircle = (zeta0 - gam^2/kappa,dot(xObs,sigma));
   //write(tObs,centerOfCircle, kappa);
   radiusOfCircle = 1.0/kappa;
}

// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++ 

pair ll = (-4.5,-2.2);
pair ur = (2.15,2.25);

draw((ll.x,0)--(ur.x,0),Arrow(2mm));
draw((0,ll.y)--(0,ur.y),Arrow(2mm));

label("$x^1$",(ur.x,0),align=S);
label("$x^2$",(0,ur.y),align=W);

real tl = 0.05;
for(int i=-4; i<=1; ++i) {
   draw((i,-tl)--(i,tl),black+0.5pt);
}
for(int i=-2; i<=2; ++i) {
   draw((-tl,i)--(tl,i),black+0.5pt);
}


real ls = 2.0;
real rodWidth = 1.5pt;
real arcWidth = 1.0pt;

tObs = -1.0;
recalcParams();
draw(graph(hyp,-ls,ls,50,operator..),black+rodWidth);
draw(circle(centerOfCircle,radiusOfCircle),blue+dashed+arcWidth);
dot(centerOfCircle);
label("\small $x_{\obs}^0=-1.0$",hyp(-ls),align=S);

tObs = -0.5;
recalcParams();
draw(graph(hyp,-ls,ls,50,operator..),black+rodWidth);
draw(circle(centerOfCircle,radiusOfCircle),blue+dashed+arcWidth);
label("\small $-0.5$",hyp(-ls),align=S);

tObs = -0.1;
recalcParams();
draw(graph(hyp,-ls,ls,50,operator..),black+rodWidth);
draw(circle(centerOfCircle,radiusOfCircle),blue+dashed+arcWidth);
label("\small $-0.1$",hyp(-ls),align=S);

tObs = 0.5;
recalcParams();
draw(graph(hyp,-ls,ls,50,operator..),black+rodWidth);
draw(circle(centerOfCircle,radiusOfCircle),red+dashed+arcWidth);
label("\small $0.5$",hyp(-ls),align=S);

tObs = 1.0;
recalcParams();
draw(graph(hyp,-ls,ls,50,operator..),black+rodWidth);
draw(arc(centerOfCircle,radiusOfCircle,-60,60),red+dashed+arcWidth);
label("\small $1.0$",hyp(-ls),align=S);

tObs = 3.0;
recalcParams();
draw(graph(hyp,-ls,ls,50,operator..),black+rodWidth);
//draw(circle(sm,1/kappa),blue+dashed);
draw(arc(centerOfCircle,radiusOfCircle,-45,45),red+dashed+arcWidth);
label("\small $3.0$",hyp(-ls),align=S);

