// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//  File:    appRodLight.asy
//  Author:  Thomas Mueller, University of Stuttgart, VISUS
//           Sebastian Boblest
//
//    Apparent view of a moving rod with light rays.
//
// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++

usepackage("amssymb");
//usepackage("mathptmx");

usepackage("format");

import geometry;
import graph;

import sr_apparent;

size(8cm, 8cm);

// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++

// observer position, time
triple xObs = ( 0.0, 0.0, 0.0 );
real   tObs = -0.5;

// offset between S and S'
triple aOff = ( 0.0, 0.0, 0.0 );

// point in S'
triple XS = (0,0,0);

// object velocity
triple beta  = (0.75,0.0,0.0);

real betaabs = 0.75;

// direction of line
triple sigma = (0.0,1.0,0.0);

int numRays = 9;

real rodLength = 4.0;
real rstep = 0.125;

real rodWidth = 1.5pt;
real arcWidth = 1.0pt;
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++ 


pair hyp( real xi ) {
    triple ax = appLinePos(aOff,xObs,beta,XS,sigma,xi,tObs);
    return (ax.x,ax.y);
}

void recalcParams() {
   real gam = 1.0/sqrt(1.0-dot(beta,beta));
   real b   = sqrt(dot(beta,beta));
   real g2  = gam*gam/(gam+1.0);
   real rho = gam*dot(XS,beta) - tObs;
   triple eta = XS + g2*dot(XS,beta)*beta + aOff - xObs;
   real   rb  = rho-dot(beta,eta);
   real   mu  = dot(eta,sigma);
   real omega = sqrt(gam*gam*rb*rb - rho*rho + dot(eta,eta) - mu^2);
   
   real zeta0 = gam^2 * b * (tObs + dot(beta,aOff-xObs)) + gam * dot(beta,XS) + dot(aOff,beta);
   real kappa = gam*b/omega;

}

real calcInitTime( real l, real tobs ){
    real w = sqrt(betaabs^2 * tobs^2 + (1-betaabs^2) * l^2);
    return -( w - tobs )/( 1 - betaabs^2 );
}
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++ 




pair ll = (-4.,-2.25);
pair ur = (1.,2.25);
int numx = 5;
int numy = 4;

// compute and draw light rays
real [] ts;
pair lr_start[];
for(int i = 0; i< numRays; ++i){
        ts[i] = calcInitTime( -rodLength*0.5 + rodLength * i * rstep, tObs );
        //rs[i].set_data([],[])
        lr_start[i] = (ts[i]*betaabs, -rodLength*0.5 + rodLength * i * rstep);
        //lr_dir[i]   = -lr_start[i]/numpy.linalg.norm(lr_start[i]);
        draw(lr_start[i]--(0,0), red+0.5bp+dashed);
}

// draw actual rod
draw((betaabs*tObs,0.5*rodLength)--(betaabs*tObs,-0.5*rodLength),blue+rodWidth);

//draw observer
dot((0,0),black+5bp);

// draw axes and labels
draw((ll.x,0)--(ur.x,0),Arrow(2mm));
draw((0,ll.y)--(0,ur.y),Arrow(2mm));

label("$x^1$",(ur.x,0),align=S);
label("$x^2$",(0,ur.y),align=0.5*W);

//draw tics
real tl = 0.05;
for(int i=-4; i<=0; ++i) {
   draw((i,-tl)--(i,tl),black+0.5pt);
}
for(int i=-2; i<=2; ++i) {
   draw((-tl,i)--(tl,i),black+0.5pt);
}


//tObs = -0.5;
recalcParams();
draw(graph(hyp,-0.5*rodLength,0.5*rodLength,50,operator..),black+rodWidth);

label("\small $x_{\obs}^0 = -0.5$",(betaabs*tObs, ll.y),align=0.2*W+S);
label("\small observer",(0,0),align=NE);
