// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//  File:    appRodView.asy
//  Author:  Thomas Mueller, University of Stuttgart, VISUS
//
//    Apparent view of a moving cuboid.
//
// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++

usepackage("amssymb");
usepackage("mathptmx");

import geometry;
import graph;

import sr_objects;
import sr_camera;
import sr_apparent;

size(8cm,0);



triple[] rodVerts = { (-0.1, -1.0, -0.1),
                      ( 0.1, -1.0, -0.1),
                      ( 0.1,  1.0, -0.1),
                      (-0.1,  1.0, -0.1),
                      (-0.1, -1.0,  0.1),
                      ( 0.1, -1.0,  0.1),
                      ( 0.1,  1.0,  0.1),
                      (-0.1,  1.0,  0.1)};

int[][] rodEdges = { {0,1}, {1,2}, {2,3}, {3,0},
                     {4,5}, {5,6}, {6,7}, {7,4},
                     {0,4}, {1,5}, {2,6}, {3,7} };
                     
real[] edgeLength = { 0.2, 2.0, 0.2, 2.0, 0.2, 2.0, 0.2, 2.0, 0.2, 0.2, 0.2, 0.2 };
int numRodEdges = rodEdges.length;

pen[] edgeColors = { 0.1bp+blue+dashed, 1bp+red, 0.1bp+blue+dashed, 0.1bp+blue+dashed, 
                     0.1bp+blue+dashed, 1bp+red, 0.1bp+blue+dashed, 0.1bp+blue+dashed, 
                     0.1bp+blue+dashed, 1bp+red, 1bp+red,           0.1bp+blue+dashed };

// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++

// observer position, time, field of view
triple xObs = ( 0.0, 0.0, 0.0 );
real   tObs = -0.1;

real   fovY   = radians(60.0);

triple dir    = ( -1.0, 0.0, 0.0 );
dir = dir/length(dir);
triple center = dir-xObs;
triple upV    = ( 0.0, 0.0, 1.0 );

// offset between S and S'
triple aOff   = ( 0.0, 0.0, 0.0 );

// object velocity
triple beta = (0.9,0.0,0.0);

// point in S'
triple XS = (0,0,0);

// direction of line
triple sigma = (1,0,0);

// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++

// View-Projection matrix
real[][] projMatrix = getPerspMatrix( fovY, 1.0, 0.1, 100.0 );
real[][] viewMatrix = getViewMatrix( xObs, center, upV );
real[][] MVP = projMatrix*viewMatrix;

// apparent line
pair appLine( real s ) {
    return map(MVP,appLinePos(aOff,xObs,beta,XS,sigma,s,tObs));
}
       
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++    
       
// minimal observer canvas       
pair ll = (-0.8,-0.333);
pair ur = (0.8,0.333);
fill(ll--(ur.x,ll.y)--ur--(ll.x,ur.y)--cycle,white);


tObs = -0.100;  
for(int i=0; i<numRodEdges; ++i) {
    XS = 0.5*(rodVerts[rodEdges[i][0]] + rodVerts[rodEdges[i][1]]);
    sigma = rodVerts[rodEdges[i][1]] - rodVerts[rodEdges[i][0]];
    sigma = sigma/length(sigma);
    draw(graph(appLine,-edgeLength[i]*0.5,edgeLength[i]*0.5,60),edgeColors[i]);
}
