// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//  File:    appSphereSingle.asy
//  Author:  Thomas Mueller, University of Stuttgart, VISUS
//
//    Apparent view of a moving sphere for close encounter.
//
// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++

usepackage("amssymb");
usepackage("mathptmx");

import geometry;
import graph;

import sr_camera;
import sr_apparent;

size(8cm,0);

// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++

// observer position, time, field of view
triple xObs = ( 0.5, 2.0, 0.0 );
real   tObs = length(xObs);

real   fovY   = radians(40.0);

// camera: point-of-interest (center)
//         and vertical-up vector
triple center = ( 0.0, 0.0, 0.0 );
triple upV    = ( 0.0, 0.0, 1.0 );

// offset between S and S'
triple aOff   = ( 0.0, 0.0, 0.0 );

// object velocity
triple beta = (0.0,0.9,0.0);

// point in S'
triple XS = (0,0,0);

// direction of line
triple sigma1 = (1,0,0);
triple sigma2 = (0,1,0);
triple sigma3 = (0,0,1);

real r = 0.5;
real Theta = pi*0.5;
real Phi   = 0.0;

// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++


// View-Projection matrix
real[][] projMatrix = getPerspMatrix( fovY, 1.0, 0.1, 100.0 );
real[][] viewMatrix = getViewMatrix( xObs, appPointPos(aOff,xObs,beta,XS,tObs), upV );
real[][] MVP = projMatrix*viewMatrix;       

pair appSphereLat( real s ) {
    triple x = appSpherePos( aOff, xObs, beta, XS, sigma1, sigma2, sigma3, r, Theta, s, tObs );
    return map(MVP,x);
}

pair appSphereLon( real s ) {
    triple x = appSpherePos( aOff, xObs, beta, XS, sigma1, sigma2, sigma3, r, s, Phi, tObs );
    return map(MVP,x);
}

// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++    
       
      
       
// minimal observer canvas       
pair ll = (-0.9,-1);
pair ur = (1.25,1);
fill(ll--(ur.x,ll.y)--ur--(ll.x,ur.y)--cycle,white);
//drawBox(MVP);

real thetaStep = radians(15.0);
real phiStep   = radians(15.0);

//tObs = 4.93;
for(real t=thetaStep; t<pi; t+=thetaStep) {
    Theta = t;
    draw(graph(appSphereLat,0,2.0*pi,100),0.1bp+blue);
}

for(real t=0; t<2.0*pi; t+=phiStep) {
    Phi = t;
    draw(graph(appSphereLon,0,pi,100),0.1bp+blue);
}
