"""
    File:    animCircle.py
    Author:  Thomas Mueller, University of Stuttgart, VISUS

    Call from within python:
            execfile("animCircle.py")
            
    The animation might perform better if started from the
    command line:
            python animCircle.py
"""

import platform
import scipy
import math
import matplotlib.pyplot as plt
from   matplotlib import animation
import sr_apparent as sr

#plt.close()

# offset between S' and S
aOff = [0,0,0]

# center point of circle
xS   = [0,0,0]

# observer position
xObs = [0,-10,0]

# velocity of S' with respect to S
beta = [0.95,0,0]

# radius of circle
rS   = 1.0

# basis vectors
sigma1 = [1,0,0]
sigma2 = [0,1,0]
sigma3 = [0,0,1]

# azimuth angle set
phiS = scipy.linspace(0,2*math.pi,100)

# initial time
tObs_start = 2.5
tObs_step  = 0.1

fig = plt.figure( figsize = (15,6), dpi=80 )
ax  = plt.axes(xlim=(-19,14.5), ylim=(-10.1,2))
ax.set_aspect('equal')
circ, = ax.plot([],[],lw=2)
time_text = ax.text(-18.5,-9.5, '')  # transform=ax.transAxes

# plot observer
plt.scatter(xObs[0],xObs[1],marker = 'D')

# plot grid
plt.grid()

pause = False

for n in range(0,13):
    pos = -17.5 + n*2.5
    c = plt.Circle((pos,0),rS,color='#dddddd')
    ax.add_patch(c)

def onClick(event):
    global pause
    pause ^= True


def init():
    circ.set_data([],[])
    time_text.set_text('')
    return circ, time_text


def animate(i):    
    tObs = tObs_start + tObs_step * i
    x1 = []
    y1 = []

    for j in range(0,scipy.size(phiS)):
        appCirc = sr.appSpherePos( aOff, xObs, beta, xS, sigma1, sigma2, sigma3, rS, math.pi/2, phiS[j], tObs )
        x1.append( appCirc[0] )
        y1.append( appCirc[1] )

    if not pause:
        circ.set_data(x1,y1)
        
    time_text.set_text("tObs = {0:4.1f}".format(tObs))    
    return circ, time_text



fig.canvas.mpl_connect('button_press_event', onClick)

doblit = True
if (platform.system()=="Darwin"):
    doblit = False
    
anim = animation.FuncAnimation( fig, animate, init_func = init, frames = 250, interval = 1, blit=doblit)

#plt.ion()
plt.show()

#anim.save('animCircle.mp4', fps=30)

# single steps:
#circ, = animate(250)
#plt.show()
