"""
    File:    appCircle.py
    Author:  Thomas Mueller, University of Stuttgart, VISUS

    Call from within python:
            execfile("appCircle.py")
    or
            run appCircle.py        
            
            
    In interactive mode, plt.ion(), use plt.close 
"""

import numpy
import scipy
import math
import matplotlib.pyplot as plt
import sr_apparent as sr

from scipy.optimize import fsolve

# offset between S' and S
aOff = [0,0,0]

# center point of circle
xS   = [0,0,0]

# observer position
xObs = [0,-10,0]

# velocity of S' with respect to S
beta = [0.95,0,0]

# radius of circle
rS   = 1.0

# basis vectors
sigma1 = [1,0,0]
sigma2 = [0,1,0]
sigma3 = [0,0,1]

# azimuth angle set
phiS = scipy.linspace(0,2*math.pi,100)

fig = plt.figure( figsize = (15,6), dpi=80 )
ax  = plt.axes( xlim=(-19,14.5), ylim=(-10.5,2.5) )
ax.set_aspect('equal')

tObs = scipy.array([3.0, 6.0, 10.0, 25.0])


def plotReference():
    x0 = []
    y0 = []

    for i in range(0,scipy.size(phiS)):
        x0.append( rS*math.cos(phiS[i]) );
        y0.append( rS*math.sin(phiS[i]) );

    # plot observer
    plt.scatter(xObs[0],xObs[1],marker = 'D')
 
    for n in range(0,13):
        pos = -17.5 + n*2.5
        xn = scipy.array(x0) + scipy.array([pos]*(scipy.size(phiS)))
        yn = scipy.array(y0)
        plt.plot(xn,yn,color='#aaaaaa')
        c = plt.Circle((pos,0),rS,color='#eeeeee')
        ax.add_patch(c)
    
    plt.grid(b=True)


def plotSpheres( to ):
    x = [ [] for i in range(scipy.size(to))]
    y = [ [] for i in range(scipy.size(to))]
   
    for i in range(0,scipy.size(phiS)):
        for j in range(0,scipy.size(to)):
            circ = sr.appSpherePos( aOff, xObs, beta, xS, sigma1, sigma2, sigma3, rS, math.pi/2, phiS[i], to[j] )
            x[j].append( circ[0] );
            y[j].append( circ[1] );
    
    for j in range(0,scipy.size(to)):
        plt.plot(x[j],y[j],'r')
   

"""
def func( phi ):
    return sr.constrEq( aOff, xObs, beta, xS, sigma1, sigma2, sigma3, rS, math.pi/2, phi, tObs[0] )

res = fsolve(func,0.1)
print res
xs = sr.appSpherePos( aOff, xObs, beta, xS, sigma1, sigma2, sigma3, rS, math.pi/2, res[0], tObs[0] )
plt.scatter(xs[0],xs[1],marker = '+')
plt.plot([xObs[0],xs[0]],[xObs[1],xs[1]],'b--')

res = fsolve(func,2.5)
print res
xs = sr.appSpherePos( aOff, xObs, beta, xS, sigma1, sigma2, sigma3, rS, math.pi/2, res[0], tObs[0] )
plt.scatter(xs[0],xs[1],marker = '+')
plt.plot([xObs[0],xs[0]],[xObs[1],xs[1]],'b--')
"""

#plt.ion();

plotReference()
plotSpheres(tObs)
plt.show()

