"""
   File:    appCube.py
   Author:  Thomas Mueller, University of Stuttgart, VISUS

   Call from within python:
            execfile("appCube.py")
"""

import numpy
import scipy
import math
import matplotlib.pyplot as plt
import sr_apparent as sr
import sr_camera   as cam

# offset between S' and S
aOff = [0,0,0]

# observer position
xObs = [ 0.5, 2, 0.0 ]

center = [0,0,0]

# observation time
tObs   = numpy.linalg.norm(xObs)

# velocity of S' with respect to S
beta = [0.0,0.9,0]

# Camera parameters
fovY = 32.0
upV  = [0,0,1]

# Construct Model-View-Projection matrix
projMatrix = cam.getPerspMatrix( fovY, 1.0, 0.1, 100.0 )
viewMatrix = cam.getViewMatrix( xObs, center, upV )
MVP = scipy.dot(projMatrix,viewMatrix)

boxVerts = scipy.array([[0,0,0],[1,0,0],[1,1,0],[0,1,0],[0,0,1],[1,0,1],[1,1,1],[0,1,1]])

boxEdges = scipy.array([[0,1],[1,2],[3,0], [4,5],[5,6],[7,4],[0,4],[1,5]])

numBoxEdges = 8

boxFrontVerts = scipy.array([[0,1,0],[1,1,0],[1,1,1],[0,1,1]])
boxFrontEdges = scipy.array([[0,1],[1,2],[2,3],[3,0]])

numBoxFrontEdges = 4

fig = plt.figure( figsize = (14,6), dpi=80 )
#ax  = plt.axes(xlim=(-0.7,0.7), ylim=(-0.7,0.7))
ax  = plt.axes(xlim=(-0.9,0.9), ylim=(-0.9,0.9))
ax.set_aspect('equal')


def plotLine( a, vel, xS, sigma, smin, smax, t, col ):    
    x = []
    y = []
    for s in scipy.linspace(smin,smax,30):
        pos = sr.appLinePos( a, xObs, vel, xS, sigma, s, t )
        npos = cam.map(MVP,pos)
        x.append(npos[0])
        y.append(npos[1])
        
    plt.plot(x,y,color=col)

    

def plotBox( a, vel, t ):
    for i in range(numBoxEdges):
        xS = 0.5*(boxVerts[boxEdges[i][0]] - scipy.array([0.5,0.5,0.5]))
        sigma = boxVerts[boxEdges[i][1]] - boxVerts[boxEdges[i][0]]
        plotLine(a,vel,xS,sigma,0,0.5,t,'b')

    for i in range(numBoxFrontEdges):
        xS = 0.5*(boxFrontVerts[boxFrontEdges[i][0]] - scipy.array([0.5,0.5,0.5]))
        sigma = boxFrontVerts[boxFrontEdges[i][1]] - boxFrontVerts[boxFrontEdges[i][0]]
        plotLine(a,vel,xS,sigma,0,0.5,t,'r')


plotBox(aOff,beta,tObs)

#plt.ion()
plt.show()

