"""
    File:    appRod.py
    Author:  Thomas Mueller, University of Stuttgart, VISUS

    Call from within python:
            execfile("appRod.py")
    or
            run appRod.py          
            
            
    Curvature and center of osculating circle calculations
    are only valid for this particular situation.
"""

import scipy
import math
import matplotlib.pyplot as plt
import sr_apparent as sr


# velocity of S' with respect to S
beta = 0.75


tObs = scipy.array([-1.0, -0.5, -0.1, 0.5, 1.0, 3.0])

#fig = plt.figure( figsize = (15,6), dpi=80 )
fig = plt.figure()
ax  = plt.axes( xlim=(-5.5,1.5), ylim=(-2.5,2.5) )
ax.set_aspect('equal')

def curvature( s, t ):
    return sr.appCurvature( [0,0,0], [0,0,0], [beta,0,0], [0,0,0], [0,1,0], s, t )


def centerOfOscCircle( t ):
    gam = 1.0/math.sqrt(1.0-beta**2)
    return gam**2 * (beta * t - math.fabs(t))


def plotRods( to ):
    x = [ [] for i in range(scipy.size(to))]
    y = [ [] for i in range(scipy.size(to))]
   
    for s in scipy.linspace(-2.0,2.0,50):
        for j in range(0,scipy.size(to)):
            rod = sr.appLinePos( [0,0,0], [0,0,0], [beta,0,0], [0,0,0], [0,1,0], s, to[j] )
            x[j].append( rod[0] );
            y[j].append( rod[1] );
    
    for j in range(0,scipy.size(to)):
        plt.plot(x[j],y[j],color='#000000',linewidth=2)
   

def plotCircle( pos, radius, phiMin, phiMax, col ):
    x = []
    y = []
    for phi in scipy.linspace(phiMin,phiMax,100):
        x.append( pos + radius*math.cos(phi) )
        y.append( radius*math.sin(phi) )
    plt.plot(x,y,'--',color=col,linewidth=2)

#plt.ion()
plt.grid(b=True)

plotCircle( centerOfOscCircle(tObs[0]), 1./curvature(0,tObs[0]), -math.pi, math.pi, '#0000ff' )
plotCircle( centerOfOscCircle(tObs[1]), 1./curvature(0,tObs[1]), -math.pi, math.pi, '#0000ff' )
plotCircle( centerOfOscCircle(tObs[2]), 1./curvature(0,tObs[2]), -math.pi, math.pi, '#0000ff' )
plotCircle( centerOfOscCircle(tObs[3]), 1./curvature(0,tObs[3]), -math.pi, math.pi, '#ff0000' )
plotCircle( centerOfOscCircle(tObs[4]), 1./curvature(0,tObs[4]), -math.pi/4, math.pi/4, '#ff0000' )
plotCircle( centerOfOscCircle(tObs[5]), 1./curvature(0,tObs[5]), -math.pi/4, math.pi/4, '#ff0000' )

plotRods(tObs)
plt.show()

