"""
   File:    appRodView.py
   Author:  Thomas Mueller, University of Stuttgart, VISUS

   Call from within python:
            execfile("appRodView.py")
"""

import numpy
import scipy
import math
import matplotlib.pyplot as plt
import sr_apparent as sr
import sr_camera   as cam

# offset between S' and S
aOff = [0,0,0]

# observer position
xObs = [0,0,0]
center = [-1,0,0]

# observation time
tObs   = -0.1

# velocity of S' with respect to S
beta = [0.9,0,0]

# Camera parameters
fovY = 60.0
upV  = [0,0,1]

# Construct Model-View-Projection matrix
projMatrix = cam.getPerspMatrix( fovY, 1.0, 0.1, 100.0 )
viewMatrix = cam.getViewMatrix( xObs, center, upV )
MVP = scipy.dot(projMatrix,viewMatrix)

rodVerts = scipy.array([[-0.1, -1.0, -0.1],[0.1, -1.0, -0.1],[0.1,  1.0, -0.1],[-0.1,  1.0, -0.1],
                        [-0.1, -1.0,  0.1],[0.1, -1.0,  0.1],[0.1,  1.0,  0.1],[-0.1,  1.0,  0.1]])
                        
rodEdges = scipy.array([[0,1],[2,3],[3,0], 
                        [4,5],[6,7],[7,4],
                        [0,4],[3,7]])

edgeLengths = scipy.array([0.2,0.2,2.0,0.2,0.2,2.0,0.2,0.2])

rodFrontEdges = scipy.array([[1,2],[5,6],[1,5],[2,6]])

edgeFrontLengths = scipy.array([2.0,2.0,0.2,0.2])

numRodEdges = 8
numRodFrontEdges = 4


fig = plt.figure( figsize = (15,6), dpi=80 )
ax  = plt.axes(xlim=(-0.8,0.8), ylim=(-0.4,0.4))
ax.set_aspect('equal')


def plotLine( a, vel, xS, sigma, smin, smax, t, col, lw ):    
    x = []
    y = []
    for s in scipy.linspace(smin,smax,100):
        pos = sr.appLinePos( a, xObs, vel, xS, sigma, s, t )
        npos = cam.map(MVP,pos)
        x.append(npos[0])
        y.append(npos[1])
        
    plt.plot(x,y,color=col,linewidth=lw)
    

def plotRod( a, vel, t ):
    for i in range(numRodEdges):
        xS = 0.5*(rodVerts[rodEdges[i][0]] + rodVerts[rodEdges[i][1]])
        sigma = (rodVerts[rodEdges[i][1]] - rodVerts[rodEdges[i][0]])
        sigma = sigma/numpy.linalg.norm(sigma)
        plotLine(a,vel,xS,sigma,-edgeLengths[i]*0.5,edgeLengths[i]*0.5,t,'b',0.5)
        
    for i in range(numRodFrontEdges):
        xS = 0.5*(rodVerts[rodFrontEdges[i][0]] + rodVerts[rodFrontEdges[i][1]])
        sigma = (rodVerts[rodFrontEdges[i][1]] - rodVerts[rodFrontEdges[i][0]])
        sigma = sigma/numpy.linalg.norm(sigma)
        plotLine(a,vel,xS,sigma,-edgeFrontLengths[i]*0.5,edgeFrontLengths[i]*0.5,t,'r',2)


plotRod(aOff,beta,tObs)

#plt.ion()
plt.show()

