"""
   File:    appSphere.py
   Author:  Thomas Mueller, University of Stuttgart, VISUS

   Call from within python:
            execfile("appSphere.py")
"""

import scipy
import math
import matplotlib.pyplot as plt
import sr_apparent as sr
import sr_camera   as cam

# offset between S' and S
aOff = [0,0,0]

# center point of circle
xS   = [0,0,0]

# observer position
xObs = [0,-10,-1.0]

# velocity of S' with respect to S
beta = [0.95,0,0]

# radius of circle
rS   = 1.0

# basis vectors
sigma1 = [0,0,1]
sigma2 = [1,0,0]
sigma3 = [0,-1,0]

# observation time
tObs   = 6.0
# indices of resting balls
cmpIdx = [4,5,6]


# Camera parameters
fovY = 40.0
upV  = [0,0,1]

# Construct Model-View-Projection matrix
projMatrix = cam.getPerspMatrix( fovY, 1.0, 0.1, 100.0 );
viewMatrix = cam.getViewMatrix( xObs, sr.appPointPos(aOff,xObs,beta,xS,tObs), upV );
MVP = scipy.dot(projMatrix,viewMatrix); 

fig = plt.figure()
ax = fig.add_subplot(111)
ax.set_aspect('equal')


# list of latitudes
thetaR = scipy.arange(15,180,15)

# list of meridians
phiR   = scipy.arange(0,360,15)


phiS   = scipy.linspace(0,2*math.pi,100)
thetaS = scipy.linspace(0,math.pi,50)


def plotSphere( vel, xSp ):
    # latitudes
    for t in range(scipy.size(thetaR)):
        theta = scipy.radians(thetaR[t])
        xp = []
        yp = []
        for phi in phiS:
            pos = sr.appSpherePos( aOff, xObs, vel, xSp, sigma1, sigma2, sigma3, rS, theta, phi, tObs )
            npos = cam.map(MVP,pos)
            xp.append(npos[0])
            yp.append(npos[1])
        
        plt.plot(xp,yp,color='b')
    
    # meridians
    for p in range(scipy.size(phiR)):
        phi = scipy.radians(phiR[p])
        xp = []
        yp = []
        for theta in thetaS:
            pos = sr.appSpherePos( aOff, xObs, vel, xSp, sigma1, sigma2, sigma3, rS, theta, phi, tObs )
            npos = cam.map(MVP,pos)
            xp.append(npos[0])
            yp.append(npos[1])
        
        plt.plot(xp,yp,color='b')


plotSphere(beta,xS)
for i in cmpIdx:
    plotSphere([0,0,0],[-17.5+2.5*i,0,-2.2])

#plt.ion()
plt.show()

