"""
   File:    appSphereSingle.py
   Author:  Thomas Mueller, University of Stuttgart, VISUS

   Call from within python:
            execfile("appSphereSingle.py")
"""

import numpy
import scipy
import math
from mpl_toolkits.mplot3d import Axes3D
import matplotlib.pyplot as plt
import sr_apparent as sr
import sr_camera   as cam

# offset between S' and S
aOff = [0,0,0]

# center point of circle
xS   = [0,0,0]

# observer position
xObs = [0.5,2.0,0.05]

# velocity of S' with respect to S
beta = [0,0.9,0]

# radius of circle
rS   = 0.5

# basis vectors
sigma1 = [1,0,0]
sigma2 = [0,1,0]
sigma3 = [0,0,1]

# observation time
tObs   = numpy.linalg.norm(numpy.array(xObs))


# Camera parameters
fovY = 40.0
upV  = [0,0,1]

# Construct Model-View-Projection matrix
projMatrix = cam.getPerspMatrix( fovY, 1.0, 0.1, 100.0 );
viewMatrix = cam.getViewMatrix( xObs, sr.appPointPos(aOff,xObs,beta,xS,tObs), upV );
MVP = scipy.dot(projMatrix,viewMatrix); 


# list of latitudes
thetaR = scipy.arange(15,180,15)

# list of meridians
phiR   = scipy.arange(0,360,15)


phiS   = scipy.linspace(0,2*math.pi,100)
thetaS = scipy.linspace(0,math.pi,100)


def plotSphere( vel, xSp ):
    # latitudes
    for t in range(scipy.size(thetaR)):
        theta = scipy.radians(thetaR[t])
        xp = []
        yp = []
        for phi in phiS:
            pos = sr.appSpherePos( aOff, xObs, vel, xSp, sigma1, sigma2, sigma3, rS, theta, phi, tObs )
            npos = cam.mapZ(MVP,pos)
            xp.append(npos[0])
            yp.append(npos[1])
            
        plt.plot(xp,yp,color='b')
    
    # meridians
    for p in range(scipy.size(phiR)):
        phi = scipy.radians(phiR[p])
        xp = []
        yp = []
        for theta in thetaS:
            pos = sr.appSpherePos( aOff, xObs, vel, xSp, sigma1, sigma2, sigma3, rS, theta, phi, tObs )
            npos = cam.mapZ(MVP,pos)
            xp.append(npos[0])
            yp.append(npos[1])
        
        plt.plot(xp,yp,color='b')


def plotSphere3D( ax, vel, xSp ):
    # latitudes
    for t in range(scipy.size(thetaR)):
        theta = scipy.radians(thetaR[t])
        xp = []
        yp = []
        zp = []
        for phi in phiS:
            pos = sr.appSpherePos( aOff, xObs, vel, xSp, sigma1, sigma2, sigma3, rS, theta, phi, tObs )
            xp.append(pos[0])
            yp.append(pos[1])
            zp.append(pos[2])
            
        ax.plot(xp,yp,zp,'b')
    
    # meridians
    
    for p in range(scipy.size(phiR)):
        phi = scipy.radians(phiR[p])
        xp = []
        yp = []
        zp = []
        for theta in thetaS:
            pos = sr.appSpherePos( aOff, xObs, vel, xSp, sigma1, sigma2, sigma3, rS, theta, phi, tObs )
            xp.append(pos[0])
            yp.append(pos[1])
            zp.append(pos[2])
        
        ax.plot(xp,yp,zp,'b')
    

fig = plt.figure()


ax = fig.add_subplot(111)
ax.set_aspect('equal')
plotSphere(beta,xS)
"""

ax = fig.gca(projection='3d')
plotSphere3D(ax, beta,xS)
plotSphere3D(ax,[0,0,0],[0,0,-1])

#xRange = [-2.0,2.0]
#Range = [-2.0,2.0]
#zRange = [-2.0,2.0]

# Create cubic bounding box to simulate equal aspect ratio
max_range = 3
Xb = 0.75*max_range*numpy.mgrid[-1:2:2,-1:2:2,-1:2:2][0].flatten() + 0.5*(numpy.array([0]))
Yb = 0.75*max_range*numpy.mgrid[-1:2:2,-1:2:2,-1:2:2][1].flatten() + 0.5*(numpy.array([0]))
Zb = 0.75*max_range*numpy.mgrid[-1:2:2,-1:2:2,-1:2:2][2].flatten() + 0.5*(numpy.array([0]))
# Comment or uncomment following both lines to test the fake bounding box:
for xb, yb, zb in zip(Xb, Yb, Zb):
   ax.plot([xb], [yb], [zb], 'w')
"""
#plt.ion()
plt.show()

