"""
   File:    demoDie.py
   Author:  Thomas Mueller, University of Stuttgart, VISUS

   Call from within python:
            execfile("demoDie.py")
"""

import platform
import scipy
import math
import matplotlib.pyplot as plt
from   matplotlib import animation
import sr_apparent as sr
import sr_camera   as cam

# offset between S' and S
aOff = [0,0,0]

# observer position
rObs = 14.422
xObs = [ 0, rObs, 2.0]
center = [0,0,0]

# observation positions
phi_start  = 0.0 #math.pi*0.5
phi_step   = 0.01
numPhiSteps  = 628

# observation times
tObs = [13.422, 14.422, 15.422]

# velocity of S' with respect to S
beta = [0,0.9,0]

# Camera parameters
fovY = 32.0
upV  = [0,0,1]

# Construct Model-View-Projection matrix
projMatrix = cam.getPerspMatrix( fovY, 1.0, 0.1, 100.0 )
viewMatrix = cam.getViewMatrix( xObs, center, upV )
MVP = scipy.dot(projMatrix,viewMatrix)

boxVerts = scipy.array([[0,0,0],[1,0,0],[1,1,0],[0,1,0],[0,0,1],[1,0,1],[1,1,1],[0,1,1]])
boxEdges = scipy.array([[0,1],[1,2],[3,0], [4,5],[5,6],[7,4],[0,4],[1,5]])

numBoxEdges = 8

boxFrontVerts = scipy.array([[0,1,0],[1,1,0],[1,1,1],[0,1,1]])
boxFrontEdges = scipy.array([[0,1],[1,2],[2,3],[3,0]])

numBoxFrontEdges = 4

fig = plt.figure( figsize = (13,6), dpi=80 )
ax  = plt.axes(xlim=(-1.0,1.0), ylim=(-0.6,0.4))
ax.set_aspect('equal')


# edges of the moving box
box = [[[] for i in range(numBoxEdges)] for j in range(0,len(tObs))]
boxFront = [[[] for i in range(numBoxFrontEdges)] for j in range(0,len(tObs))]

for j in range(0,len(tObs)):
    for i in range(numBoxEdges):
        box[j][i], = ax.plot([],[],'b')
    for i in range(numBoxFrontEdges):
        boxFront[j][i], = ax.plot([],[],'r')

boxTextList = ()

numStaticBoxes = 8
staticBox = [[[] for i in range(numBoxEdges)] for j in range(numStaticBoxes)]
staticBoxFront = [[[] for i in range(numBoxFrontEdges)] for j in range(numStaticBoxes)]
    
for j in range(numStaticBoxes):
    for i in range(numBoxEdges):
        staticBox[j][i], = ax.plot([],[],'b')
    for i in range(numBoxFrontEdges):
        staticBoxFront[j][i], = ax.plot([],[],'r')
    


def init():
    global boxTextList
    
    for j in range(0,len(tObs)):
        for i in range(numBoxEdges):
            box[j][i].set_data([],[])
            boxTextList += (box[j][i],)
        for i in range(numBoxFrontEdges):
            boxFront[j][i].set_data([],[])
            boxTextList += (boxFront[j][i],)
            
    for j in range(numStaticBoxes):
        for i in range(numBoxEdges):
            staticBox[j][i].set_data([],[])
            boxTextList += (staticBox[j][i],)
        for i in range(numBoxFrontEdges):
            staticBoxFront[j][i].set_data([],[])
            boxTextList += (staticBoxFront[j][i],)
            
    return boxTextList
    
    

def animate(i):           
    phi = phi_start + i*phi_step
    xObs = [ rObs*math.cos(phi), rObs*math.sin(phi), 2.0]
    
    viewMatrix = cam.getViewMatrix( xObs, center, upV )
    MVP = scipy.dot(projMatrix,viewMatrix)
    
    for k in range(0,len(tObs)):
        for j in range(numBoxEdges):
            xS = 0.5*(boxVerts[boxEdges[j][0]] - scipy.array([0.5,0.5,0.5]))
            sigma = boxVerts[boxEdges[j][1]] - boxVerts[boxEdges[j][0]]
                
            x = []
            y = []
            for s in scipy.linspace(0,0.5,20):
                pos = sr.appLinePos( aOff, xObs, beta, xS, sigma, s, tObs[k] )
                npos = cam.map(MVP,pos)
                x.append(npos[0])
                y.append(npos[1])
                
            box[k][j].set_data(x,y)
    
        for j in range(numBoxFrontEdges):
            xS = 0.5*(boxFrontVerts[boxFrontEdges[j][0]] - scipy.array([0.5,0.5,0.5]))
            sigma = boxFrontVerts[boxFrontEdges[j][1]] - boxFrontVerts[boxFrontEdges[j][0]]
                
            x = []
            y = []
            for s in scipy.linspace(0,0.5,20):
                pos = sr.appLinePos( aOff, xObs, beta, xS, sigma, s, tObs[k] )
                npos = cam.map(MVP,pos)
                x.append(npos[0])
                y.append(npos[1])
                
            boxFront[k][j].set_data(x,y)
    
    for n in range(numStaticBoxes):
        a = [0,-10+n*2,-0.75]
        for j in range(numBoxEdges):
            vA = 0.5*(boxVerts[boxEdges[j][0]] - scipy.array([0.5,0.5,0.5])) + a
            vB = 0.5*(boxVerts[boxEdges[j][1]] - scipy.array([0.5,0.5,0.5])) + a
            posA = cam.map(MVP,vA)
            posB = cam.map(MVP,vB)
            x = [posA[0],posB[0]]
            y = [posA[1],posB[1]]
            staticBox[n][j].set_data(x,y)
        
        for j in range(numBoxFrontEdges):
            vA = 0.5*(boxFrontVerts[boxFrontEdges[j][0]] - scipy.array([0.5,0.5,0.5])) + a
            vB = 0.5*(boxFrontVerts[boxFrontEdges[j][1]] - scipy.array([0.5,0.5,0.5])) + a
            posA = cam.map(MVP,vA)
            posB = cam.map(MVP,vB)
            x = [posA[0],posB[0]]
            y = [posA[1],posB[1]]
            staticBoxFront[n][j].set_data(x,y)
    
    return boxTextList

doblit = True
if (platform.system()=="Darwin"):
    doblit = False
    
anim = animation.FuncAnimation( fig, animate, init_func = init, frames = numPhiSteps, interval = 1, blit=doblit)
plt.show()
